﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace _762000_TreeViewsDragDrop
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            BindingList<Product> products = new BindingList<Product>();
            products.Add(new Product(567, "Bicycle", 5));
            products.Add(new Product(456, "Car", 5000));
            products.Add(new Product(789, "Bike", 1500));
            products.Add(new Product(754, "Motor-cycle", 1500));
            radTreeView1.DataSource = products;
            radTreeView1.DisplayMember = "Description";

            this.radTreeView1.AllowDragDrop = true;
            //this.radTreeView1.MultiSelect = true;
           
            BindingList<Product> products2 = new BindingList<Product>();
            products2.Add(new Product(567, "T-shirt", 5));
            products2.Add(new Product(456, "Jacket", 5000));
            products2.Add(new Product(789, "Jeans", 1500));  
            this.radTreeView2.DataSource = products2;
            this.radTreeView2.DisplayMember = "Description";
        }
    }

    class CustomTreeView : RadTreeView
    {
        protected override RadTreeViewElement CreateTreeViewElement()
        {
            return new CustomTreeViewElement();
        }

        public override string ThemeClassName
        {
            get
            {
                return typeof(RadTreeView).FullName;
            }
        }
    }
    
    class CustomTreeViewElement : RadTreeViewElement
    {
        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(RadTreeViewElement);
            }
        }

        protected override TreeViewDragDropService CreateDragDropService()
        {
            return new CustomDragDropService(this);
        }
    }

    class CustomDragDropService : TreeViewDragDropService
    {
        RadTreeViewElement owner;
        RadTreeNode draggedNode;

        public CustomDragDropService(RadTreeViewElement owner) : base(owner)
        {
            this.owner = owner;
        }

        //Save the dragged node
        protected override void PerformStart()
        {
            base.PerformStart();

            TreeNodeElement draggedNodeElement = this.Context as TreeNodeElement;
            this.draggedNode = draggedNodeElement.Data;
        }

        //Clean the saved node
        protected override void PerformStop()
        {
            base.PerformStop();
            this.draggedNode = null;
        }

        protected override void OnPreviewDragHint(PreviewDragHintEventArgs e)
        {
            
            base.OnPreviewDragHint(e);
        }
        //If tree element is hovered, allow drop
        protected override void OnPreviewDragOver(RadDragOverEventArgs e)
        {
            base.OnPreviewDragOver(e);

            RadTreeViewElement targetElement = e.HitTarget as RadTreeViewElement;
            if (targetElement != null && targetElement != this.owner)
            {
                e.CanDrop = true;
            }
            else if (e.HitTarget is TreeNodeElement)
            {
                e.CanDrop = true;
            }
        }

        //Create copies of the selected node(s) and add them to the hovered node/tree
        protected override void OnPreviewDragDrop(RadDropEventArgs e)
        {
            TreeNodeElement targetNodeElement = e.HitTarget as TreeNodeElement;
            RadTreeViewElement targetTreeView = targetNodeElement == null ? e.HitTarget as RadTreeViewElement : targetNodeElement.TreeViewElement;

            if (targetTreeView == this.owner)
            {
                base.OnPreviewDragDrop(e);
                return;
            }

            if (targetTreeView == null)
            {
                return;
            }

            List<RadTreeNode> draggedNodes = this.GetDraggedNodes(draggedNode);
            
            targetTreeView.BeginUpdate();
            this.owner.BeginUpdate();
            
            foreach (RadTreeNode node in draggedNodes)
            {
                Product product = node.DataBoundItem as Product;
                BindingList<Product> targetList = targetTreeView.DataSource as BindingList<Product>;
                if (!targetList.Contains(product))
                {
                    if (targetNodeElement != null)
                    {
                        RadTreeNode nodeOver = targetNodeElement.Data;
                        int insertIndex = targetList.IndexOf(nodeOver.DataBoundItem as Product);
                        if (insertIndex < targetList.Count)
                        {
                            targetList.Insert(insertIndex, product);
                        }
                        else
                        {
                            targetList.Add(product);
                        }
                    }
                    else
                    {
                        targetList.Add(product);
                    }
                }
            }

            this.owner.EndUpdate();
            targetTreeView.EndUpdate();
        }
    }
     
    public class Product
    {
        private int _id;
        private string _description;
        private float _price;

        public int ID
        {
            get
            {
                return _id;
            }
            set
            {
                _id = value;
            }
        }

        public string Description
        {
            get
            {
                return _description;
            }
            set
            {
                _description = value;
            }
        }

        public float Price
        {
            get
            {
                return _price;
            }
            set
            {
                _price = value;
            }
        }

        public Product(int id, string description, float price)
        {
            _id = id;
            _description = description;
            _price = price;
        }
    }
}